@@ -45,13 +45,6 @@ if 1:
             plt.nipy_spectral()
         except AttributeError:
             plt.spectral()
-        try:
-            import signal
-            def action(*args): raise SystemExit
-            signal.signal(signal.SIGALRM, action)
-            signal.alarm(2)
-        except:
-            pass
-        plt.show()
+        plt.pause(2)
     except:
         pass
