@@ -4,7 +4,7 @@ from mpi4py import MPI
 from array import array
 from threading import Thread
 
-class Counter(object):
+class Counter:
 
     def __init__(self, comm):
         # duplicate communicator
@@ -27,8 +27,8 @@ class Counter(object):
                            status)
             if status.Get_tag() == 1:
                 return
-            self.comm.Send([ival, MPI.INT],
-                           status.Get_source(), 0)
+            self.comm.Ssend([ival, MPI.INT],
+                            status.Get_source(), 0)
             ival[0] += incr[0]
 
     def free(self):
@@ -36,7 +36,7 @@ class Counter(object):
         # stop counter thread
         rank = self.comm.Get_rank()
         if rank == 0:
-            self.comm.Send([None, MPI.INT], 0, 1)
+            self.comm.Ssend([None, MPI.INT], 0, 1)
             self.thread.join()
         #
         self.comm.Free()
@@ -44,7 +44,7 @@ class Counter(object):
     def next(self):
         incr = array('i', [1])
         ival = array('i', [0])
-        self.comm.Send([incr, MPI.INT], 0, 0)
+        self.comm.Ssend([incr, MPI.INT], 0, 0)
         self.comm.Recv([ival, MPI.INT], 0, 0)
         nxtval = ival[0]
         return nxtval
