@@ -24,7 +24,7 @@ def osu_barrier(
         raise SystemExit(errmsg)
 
     if myid == 0:
-        print ('# %s' % (BENCHMARH,))
+        print (f'# {BENCHMARH}')
     if myid == 0:
         print ('# %-8s%20s' % ("Size [B]", "Latency [us]"))
 
