@@ -1,5 +1,3 @@
-#import mpi4py
-#mpi4py.profile("mpe")
 from mpi4py import MPI
 
 import unittest
@@ -9,7 +7,7 @@ sys.path.insert(0, os.path.dirname(__file__))
 from reductions import Intracomm
 del sys.path[0]
 
-class BaseTest(object):
+class BaseTest:
 
     def test_reduce(self):
         rank = self.comm.rank
@@ -20,7 +18,7 @@ class BaseTest(object):
             if self.comm.rank == root:
                 self.assertEqual(res, sum(range(size)))
             else:
-                self.assertEqual(res, None)
+                self.assertIsNone(res)
 
     def test_reduce_min(self):
         rank = self.comm.rank
@@ -31,7 +29,7 @@ class BaseTest(object):
             if self.comm.rank == root:
                 self.assertEqual(res, 0)
             else:
-                self.assertEqual(res, None)
+                self.assertIsNone(res)
 
     def test_reduce_max(self):
         rank = self.comm.rank
@@ -42,7 +40,7 @@ class BaseTest(object):
             if self.comm.rank == root:
                 self.assertEqual(res, size-1)
             else:
-                self.assertEqual(res, None)
+                self.assertIsNone(res)
 
     def test_reduce_minloc(self):
         rank = self.comm.rank
@@ -53,7 +51,7 @@ class BaseTest(object):
             if self.comm.rank == root:
                 self.assertEqual(res, (0, 0))
             else:
-                self.assertEqual(res, None)
+                self.assertIsNone(res)
 
     def test_reduce_maxloc(self):
         rank = self.comm.rank
@@ -64,7 +62,7 @@ class BaseTest(object):
             if self.comm.rank == root:
                 self.assertEqual(res, (size-1, size-1))
             else:
-                self.assertEqual(res, None)
+                self.assertIsNone(res)
 
     def test_allreduce(self):
         rank = self.comm.rank
@@ -142,7 +140,7 @@ class BaseTest(object):
         msg = rank
         res = self.comm.exscan(sendobj=msg)
         if self.comm.rank == 0:
-            self.assertEqual(res, None)
+            self.assertIsNone(res)
         else:
             self.assertEqual(res, sum(list(range(size))[:rank]))
 
@@ -152,7 +150,7 @@ class BaseTest(object):
         msg = rank
         res = self.comm.exscan(sendobj=msg, op=MPI.MIN)
         if self.comm.rank == 0:
-            self.assertEqual(res, None)
+            self.assertIsNone(res)
         else:
             self.assertEqual(res, 0)
 
@@ -162,7 +160,7 @@ class BaseTest(object):
         msg = rank
         res = self.comm.exscan(sendobj=msg, op=MPI.MAX)
         if self.comm.rank == 0:
-            self.assertEqual(res, None)
+            self.assertIsNone(res)
         else:
             self.assertEqual(res, rank-1)
 
@@ -172,7 +170,7 @@ class BaseTest(object):
         msg = rank
         res = self.comm.exscan(sendobj=(msg, rank), op=MPI.MINLOC)
         if self.comm.rank == 0:
-            self.assertEqual(res, None)
+            self.assertIsNone(res)
         else:
             self.assertEqual(res, (0, 0))
 
@@ -182,7 +180,7 @@ class BaseTest(object):
         msg = rank
         res = self.comm.exscan(sendobj=(msg, rank), op=MPI.MAXLOC)
         if self.comm.rank == 0:
-            self.assertEqual(res, None)
+            self.assertIsNone(res)
         else:
             self.assertEqual(res, (rank-1, rank-1))
 
