@@ -12,8 +12,11 @@
 #endif
 
 /* MPI I/O may not be available */
+/* https://github.com/pmodels/mpich/issues/7278 */
+#if MPICH_NUMVERSION < 40300000
 #ifndef ROMIO_VERSION
 #include "mpi-io.h"
 #endif
+#endif
 
 #endif /* !PyMPI_CONFIG_MPICH_H */
