@@ -6,6 +6,8 @@ from functools import reduce
 prod = lambda sequence,start=1: reduce(lambda x, y: x*y, sequence, start)
 
 def skip_op(typecode, op):
+    if typecode in '?':
+        return True
     if typecode in 'FDG':
         if op in (MPI.MAX, MPI.MIN):
             return True
@@ -25,7 +27,7 @@ def maxvalue(a):
 
 
 @unittest.skipMPI('msmpi(<8.1.0)')
-class BaseTestCCOBuf(object):
+class BaseTestCCOBuf:
 
     COMM = MPI.COMM_NULL
 
@@ -35,21 +37,23 @@ class BaseTestCCOBuf(object):
     def testBcast(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 if rank == root:
                     buf = array(root, typecode, root)
                 else:
                     buf = array(  -1, typecode, root)
                 self.COMM.Ibcast(buf.as_mpi(), root=root).Wait()
                 for value in buf:
-                    self.assertEqual(value, root)
+                    self.assertEqual(value, check)
 
     def testGather(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 sbuf = array(root, typecode, root+1)
                 if rank == root:
                     rbuf = array(-1, typecode, (size,root+1))
@@ -59,13 +63,14 @@ class BaseTestCCOBuf(object):
                                   root=root).Wait()
                 if rank == root:
                     for value in rbuf.flat:
-                        self.assertEqual(value, root)
+                        self.assertEqual(value, check)
 
     def testScatter(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 rbuf = array(-1, typecode, size)
                 if rank == root:
                     sbuf = array(root, typecode, (size, size))
@@ -74,40 +79,36 @@ class BaseTestCCOBuf(object):
                 self.COMM.Iscatter(sbuf.as_mpi(), rbuf.as_mpi(),
                                    root=root).Wait()
                 for value in rbuf:
-                    self.assertEqual(value, root)
+                    self.assertEqual(value, check)
 
     def testAllgather(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 sbuf = array(root, typecode, root+1)
                 rbuf = array(  -1, typecode, (size, root+1))
                 self.COMM.Iallgather(sbuf.as_mpi(), rbuf.as_mpi()).Wait()
                 for value in rbuf.flat:
-                    self.assertEqual(value, root)
+                    self.assertEqual(value, check)
 
     def testAlltoall(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 sbuf = array(root, typecode, (size, root+1))
                 rbuf = array(  -1, typecode, (size, root+1))
                 self.COMM.Ialltoall(sbuf.as_mpi(), rbuf.as_mpi_c(root+1)).Wait()
                 for value in rbuf.flat:
-                    self.assertEqual(value, root)
-
-    def assertAlmostEqual(self, first, second):
-        num = complex(second-first)
-        den = complex(second+first)/2 or 1.0
-        if (abs(num/den) > 1e-2):
-            raise self.failureException('%r != %r' % (first, second))
+                    self.assertEqual(value, check)
 
     def testReduce(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
@@ -121,7 +122,8 @@ class BaseTestCCOBuf(object):
                     max_val = maxvalue(rbuf)
                     for i, value in enumerate(rbuf):
                         if rank != root:
-                            self.assertEqual(value, -1)
+                            check = arrayimpl.scalar(-1)
+                            self.assertEqual(value, check)
                             continue
                         if op == MPI.SUM:
                             if (i * size) < max_val:
@@ -137,7 +139,7 @@ class BaseTestCCOBuf(object):
     def testAllreduce(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.MAX, MPI.MIN, MPI.PROD):
@@ -164,7 +166,7 @@ class BaseTestCCOBuf(object):
     def testReduceScatter(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.MAX, MPI.MIN, MPI.PROD):
@@ -211,7 +213,7 @@ class BaseTestCCOBuf(object):
     def testReduceScatterBlock(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.MAX, MPI.MIN, MPI.PROD):
@@ -246,7 +248,7 @@ class BaseTestCCOBuf(object):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
         # --
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
@@ -273,7 +275,7 @@ class BaseTestCCOBuf(object):
     def testExscan(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
@@ -303,7 +305,7 @@ class BaseTestCCOBuf(object):
     def testBcastTypeIndexed(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             datatype = array.TypeMap[typecode]
             for root in range(size):
                 #
@@ -319,10 +321,8 @@ class BaseTestCCOBuf(object):
                 newtype.Free()
                 if rank != root:
                     for i, value in enumerate(buf):
-                        if (i % 2):
-                            self.assertEqual(value, -1)
-                        else:
-                            self.assertEqual(value, i)
+                        check = arrayimpl.scalar(-1 if (i % 2) else i)
+                        self.assertEqual(value, check)
 
                 #
                 if rank == root:
@@ -337,21 +337,19 @@ class BaseTestCCOBuf(object):
                 newtype.Free()
                 if rank != root:
                     for i, value in enumerate(buf):
-                        if not (i % 2):
-                            self.assertEqual(value, -1)
-                        else:
-                            self.assertEqual(value, i)
+                        check = arrayimpl.scalar(-1 if not (i % 2) else i)
+                        self.assertEqual(value, check)
 
 
-@unittest.skipMPI('MVAPICH2')
 @unittest.skipMPI('msmpi(<8.1.0)')
-class BaseTestCCOBufInplace(object):
+class BaseTestCCOBufInplace:
 
     def testGather(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 count = root+3
                 if rank == root:
                     sbuf = MPI.IN_PLACE
@@ -359,7 +357,7 @@ class BaseTestCCOBufInplace(object):
                     #buf.flat[(rank*count):((rank+1)*count)] = \
                     #    array(root, typecode, count)
                     s, e = rank*count, (rank+1)*count
-                    for i in range(s, e): buf.flat[i] = root
+                    for i in range(s, e): buf.flat[i] = check
                     rbuf = buf.as_mpi()
                 else:
                     buf = array(root, typecode, count)
@@ -367,14 +365,20 @@ class BaseTestCCOBufInplace(object):
                     rbuf = None
                 self.COMM.Igather(sbuf, rbuf, root=root).Wait()
                 for value in buf.flat:
-                    self.assertEqual(value, root)
+                    self.assertEqual(value, check)
+                if rank == root:
+                    sbuf = None
+                self.COMM.Igather(sbuf, rbuf, root=root).Wait()
+                for value in buf.flat:
+                    self.assertEqual(value, check)
 
     @unittest.skipMPI('msmpi(==10.0.0)')
     def testScatter(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 for count in range(1, 10):
                     if rank == root:
                         buf = array(root, typecode, (size, count))
@@ -386,38 +390,39 @@ class BaseTestCCOBufInplace(object):
                         rbuf = buf.as_mpi()
                     self.COMM.Iscatter(sbuf, rbuf, root=root).Wait()
                     for value in buf.flat:
-                        self.assertEqual(value, root)
+                        self.assertEqual(value, check)
+                    if rank == root:
+                        rbuf = None
+                    self.COMM.Iscatter(sbuf, rbuf, root=root).Wait()
+                    for value in buf.flat:
+                        self.assertEqual(value, check)
+
 
     def testAllgather(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for count in range(1, 10):
+                check = arrayimpl.scalar(count)
                 buf = array(-1, typecode, (size, count))
-                #buf.flat[(rank*count):((rank+1)*count)] = \
-                #    array(count, typecode, count)
                 s, e = rank*count, (rank+1)*count
-                for i in range(s, e): buf.flat[i] = count
+                for i in range(s, e): buf.flat[i] = check
                 self.COMM.Iallgather(MPI.IN_PLACE, buf.as_mpi()).Wait()
                 for value in buf.flat:
-                    self.assertEqual(value, count)
-
-    def assertAlmostEqual(self, first, second):
-        num = complex(second-first)
-        den = complex(second+first)/2 or 1.0
-        if (abs(num/den) > 1e-2):
-            raise self.failureException('%r != %r' % (first, second))
+                    self.assertEqual(value, check)
+                self.COMM.Iallgather(None, buf.as_mpi()).Wait()
+                for value in buf.flat:
+                    self.assertEqual(value, check)
 
     def testReduce(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
                 if skip_op(typecode, op): continue
                 for root in range(size):
-                    count = size
                     if rank == root:
                         buf  = array(range(size), typecode)
                         sbuf = MPI.IN_PLACE
@@ -445,7 +450,7 @@ class BaseTestCCOBufInplace(object):
     def testAllreduce(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.MAX, MPI.MIN, MPI.PROD):
@@ -471,7 +476,7 @@ class BaseTestCCOBufInplace(object):
     def testReduceScatterBlock(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.MAX, MPI.MIN, MPI.PROD):
@@ -509,7 +514,7 @@ class BaseTestCCOBufInplace(object):
     def testReduceScatter(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.MAX, MPI.MIN, MPI.PROD):
@@ -548,7 +553,7 @@ class BaseTestCCOBufInplace(object):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
         # --
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
@@ -574,7 +579,7 @@ class BaseTestCCOBufInplace(object):
     def testExscan(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             # segfault as of OpenMPI 4.1.1
             if unittest.is_mpi_gpu('openmpi', array): continue
             for op in (MPI.SUM, MPI.PROD, MPI.MAX, MPI.MIN):
@@ -615,9 +620,9 @@ class TestCCOBufInplaceSelf(BaseTestCCOBufInplace, unittest.TestCase):
 
 class TestCCOBufInplaceWorld(BaseTestCCOBufInplace, unittest.TestCase):
     COMM = MPI.COMM_WORLD
-    @unittest.skipMPI('IntelMPI', MPI.COMM_WORLD.Get_size() > 1)
+    @unittest.skipMPI('mvapich(<3.0.0)', MPI.COMM_WORLD.Get_size() > 1)
     def testReduceScatter(self):
-        super(TestCCOBufInplaceWorld, self).testReduceScatter()
+        super().testReduceScatter()
 
 class TestCCOBufSelfDup(TestCCOBufSelf):
     def setUp(self):
