@@ -18,15 +18,17 @@ def maxvalue(a):
 
 @unittest.skipMPI('msmpi(<8.1.0)')
 @unittest.skipMPI('openmpi(<2.0.0)')
-class BaseTestCCOVec(object):
+class BaseTestCCOVec:
 
     COMM = MPI.COMM_NULL
 
     def testGatherv(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, count)
                     rbuf = array(  -1, typecode, size*size)
@@ -40,15 +42,17 @@ class BaseTestCCOVec(object):
                             row = rbuf[i*size:(i+1)*size]
                             a, b = row[:count], row[count:]
                             for va in a:
-                                self.assertEqual(va, root)
+                                self.assertEqual(va, check_a)
                             for vb in b:
-                                self.assertEqual(vb, -1)
+                                self.assertEqual(vb, check_b)
 
     def testGatherv2(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, size)
                     rbuf = array(  -1, typecode, size*size)
@@ -61,15 +65,16 @@ class BaseTestCCOVec(object):
                             row = rbuf[i*size:(i+1)*size]
                             a, b = row[:count], row[count:]
                             for va in a:
-                                self.assertEqual(va, root)
+                                self.assertEqual(va, check_a)
                             for vb in b:
-                                self.assertEqual(vb, -1)
+                                self.assertEqual(vb, check_b)
 
     def testGatherv3(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 for count in range(size+1):
                     #
                     sbuf = array(root, typecode, count).as_raw()
@@ -80,7 +85,7 @@ class BaseTestCCOVec(object):
                     self.COMM.Igatherv(sendbuf, recvbuf, root).Wait()
                     if recvbuf is not None:
                         for v in rbuf:
-                            self.assertEqual(v, root)
+                            self.assertEqual(v, check)
                     #
                     sbuf = array(root, typecode, count).as_raw()
                     if rank == root:
@@ -91,13 +96,14 @@ class BaseTestCCOVec(object):
                     self.COMM.Barrier()
                     if rank == root:
                         for v in rbuf:
-                            self.assertEqual(v, root)
+                            self.assertEqual(v, check)
 
     def testScatterv(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 for count in range(size):
                     sbuf = array(root, typecode, size*size)
                     rbuf = array(  -1, typecode, count)
@@ -107,13 +113,15 @@ class BaseTestCCOVec(object):
                     if rank != root: sendbuf = None
                     self.COMM.Iscatterv(sendbuf, rbuf.as_mpi(), root).Wait()
                     for vr in rbuf:
-                        self.assertEqual(vr, root)
+                        self.assertEqual(vr, check)
 
     def testScatterv2(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, size*size)
                     rbuf = array(  -1, typecode, size)
@@ -123,15 +131,16 @@ class BaseTestCCOVec(object):
                     self.COMM.Iscatterv(sendbuf, recvbuf, root).Wait()
                     a, b = rbuf[:count], rbuf[count:]
                     for va in a:
-                        self.assertEqual(va, root)
+                        self.assertEqual(va, check_a)
                     for vb in b:
-                        self.assertEqual(vb, -1)
+                        self.assertEqual(vb, check_b)
 
     def testScatterv3(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 for count in range(size+1):
                     #
                     sbuf = array(root, typecode, count*size).as_raw()
@@ -141,7 +150,7 @@ class BaseTestCCOVec(object):
                     if rank != root: sendbuf = None
                     self.COMM.Iscatterv(sendbuf, recvbuf, root).Wait()
                     for v in rbuf:
-                        self.assertEqual(v, root)
+                        self.assertEqual(v, check)
                     #
                     if rank == root:
                         sbuf = array(root, typecode, count*size).as_raw()
@@ -150,13 +159,15 @@ class BaseTestCCOVec(object):
                     rbuf = array(  -1, typecode, count).as_raw()
                     self.COMM.Scatterv(sbuf, rbuf, root)
                     for v in rbuf:
-                        self.assertEqual(v, root)
+                        self.assertEqual(v, check)
 
     def testAllgatherv(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, count)
                     rbuf = array(  -1, typecode, size*size)
@@ -169,15 +180,17 @@ class BaseTestCCOVec(object):
                         row = rbuf[i*size:(i+1)*size]
                         a, b = row[:count], row[count:]
                         for va in a:
-                            self.assertEqual(va, root)
+                            self.assertEqual(va, check_a)
                         for vb in b:
-                            self.assertEqual(vb, -1)
+                            self.assertEqual(vb, check_b)
 
     def testAllgatherv2(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, size)
                     rbuf = array(  -1, typecode, size*size)
@@ -188,15 +201,16 @@ class BaseTestCCOVec(object):
                         row = rbuf[i*size:(i+1)*size]
                         a, b = row[:count], row[count:]
                         for va in a:
-                            self.assertEqual(va, root)
+                            self.assertEqual(va, check_a)
                         for vb in b:
-                            self.assertEqual(vb, -1)
+                            self.assertEqual(vb, check_b)
 
     def testAllgatherv3(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 for count in range(size+1):
                     #
                     sbuf = array(root, typecode, count).as_raw()
@@ -205,19 +219,21 @@ class BaseTestCCOVec(object):
                     recvbuf = [rbuf, count]
                     self.COMM.Iallgatherv(sendbuf, recvbuf).Wait()
                     for v in rbuf:
-                        self.assertEqual(v, root)
+                        self.assertEqual(v, check)
                     #
                     sbuf = array(root, typecode, count).as_raw()
                     rbuf = array(  -1, typecode, count*size).as_raw()
                     self.COMM.Iallgatherv(sbuf, rbuf).Wait()
                     for v in rbuf:
-                        self.assertEqual(v, root)
+                        self.assertEqual(v, check)
 
     def testAlltoallv(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, size*size)
                     rbuf = array(  -1, typecode, size*size)
@@ -230,15 +246,17 @@ class BaseTestCCOVec(object):
                         row = rbuf[i*size:(i+1)*size]
                         a, b = row[:count], row[count:]
                         for va in a:
-                            self.assertEqual(va, root)
+                            self.assertEqual(va, check_a)
                         for vb in b:
-                            self.assertEqual(vb, -1)
+                            self.assertEqual(vb, check_b)
 
     def testAlltoallv2(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check_a = arrayimpl.scalar(root)
+                check_b = arrayimpl.scalar(-1)
                 for count in range(size):
                     sbuf = array(root, typecode, size*size)
                     rbuf = array(  -1, typecode, size*size)
@@ -249,15 +267,16 @@ class BaseTestCCOVec(object):
                         row = rbuf[i*size:(i+1)*size]
                         a, b = row[:count], row[count:]
                         for va in a:
-                            self.assertEqual(va, root)
+                            self.assertEqual(va, check_a)
                         for vb in b:
-                            self.assertEqual(vb, -1)
+                            self.assertEqual(vb, check_b)
 
     def testAlltoallv3(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for root in range(size):
+                check = arrayimpl.scalar(root)
                 for count in range(size+1):
                     #
                     sbuf = array(root, typecode, count*size).as_raw()
@@ -266,19 +285,20 @@ class BaseTestCCOVec(object):
                     recvbuf = [rbuf, count]
                     self.COMM.Ialltoallv(sendbuf, recvbuf).Wait()
                     for v in rbuf:
-                        self.assertEqual(v, root)
+                        self.assertEqual(v, check)
                     #
                     sbuf = array(root, typecode, count*size).as_raw()
                     rbuf = array(  -1, typecode, count*size).as_raw()
                     self.COMM.Ialltoallv(sbuf, rbuf).Wait()
                     for v in rbuf:
-                        self.assertEqual(v, root)
+                        self.assertEqual(v, check)
 
     def testAlltoallw(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for n in range(1, size+1):
+                check = arrayimpl.scalar(n)
                 sbuf = array( n, typecode, (size, n))
                 rbuf = array(-1, typecode, (size, n))
                 sdt, rdt = sbuf.mpidtype, rbuf.mpidtype
@@ -289,21 +309,52 @@ class BaseTestCCOVec(object):
                 try:
                     self.COMM.Ialltoallw(smsg, rmsg).Wait()
                 except NotImplementedError:
-                    self.skipTest('mpi-ialltoallw')
-                for v in rbuf.flat:
-                    self.assertEqual(v, n)
+                    self.skipTest('mpi-alltoallw')
+                for value in rbuf.flat:
+                    self.assertEqual(value, check)
 
+    @unittest.skipMPI('openmpi(<4.1.7)')
+    def testAlltoallwBottom(self):
+        size = self.COMM.Get_size()
+        rank = self.COMM.Get_rank()
+        for array, typecode in arrayimpl.loop():
+            for n in range(1, size+1):
+                check = arrayimpl.scalar(n)
+                sbuf = array( n, typecode, (size, n))
+                rbuf = array(-1, typecode, (size, n))
+                saddr = MPI.Get_address(sbuf.as_raw())
+                raddr = MPI.Get_address(rbuf.as_raw())
+                sdt, rdt = sbuf.mpidtype, rbuf.mpidtype
+                stypes = [
+                    MPI.Datatype.Create_struct([n], [saddr+d], [sdt]).Commit()
+                    for d in list(range(0, size*n*sdt.extent, n*sdt.extent))
+                ]
+                rtypes = [
+                    MPI.Datatype.Create_struct([n], [raddr+d], [sdt]).Commit()
+                    for d in list(range(0, size*n*rdt.extent, n*rdt.extent))
+                ]
+                smsg = (MPI.BOTTOM, ([1]*size, [0]*size), stypes)
+                rmsg = (MPI.BOTTOM, ([1]*size, [0]*size), rtypes)
+                try:
+                    self.COMM.Ialltoallw(smsg, rmsg).Wait()
+                except NotImplementedError:
+                    self.skipTest('mpi-alltoallw')
+                finally:
+                    for t in stypes: t.Free()
+                    for t in rtypes: t.Free()
+                for value in rbuf.flat:
+                    self.assertEqual(value, check)
 
 @unittest.skipMPI('msmpi(<8.1.0)')
 @unittest.skipMPI('openmpi(<2.0.0)')
-class BaseTestCCOVecInplace(object):
+class BaseTestCCOVecInplace:
 
     COMM = MPI.COMM_NULL
 
     def testAlltoallv(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for count in range(size):
                 rbuf = array(-1, typecode, size*size)
                 counts = [count] * size
@@ -317,14 +368,16 @@ class BaseTestCCOVecInplace(object):
                     row = rbuf[i*size:(i+1)*size]
                     a, b = row[:count], row[count:]
                     for va in a:
-                        self.assertEqual(va, i)
+                        check_a = arrayimpl.scalar(i)
+                        self.assertEqual(va, check_a)
                     for vb in b:
-                        self.assertEqual(vb, -1)
+                        check_b = arrayimpl.scalar(-1)
+                        self.assertEqual(vb, check_b)
 
     def testAlltoallw(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for count in range(size):
                 rbuf = array(-1, typecode, size*size)
                 for i in range(size):
@@ -341,14 +394,16 @@ class BaseTestCCOVecInplace(object):
                     row = rbuf[i*size:(i+1)*size]
                     a, b = row[:count], row[count:]
                     for va in a:
-                        self.assertEqual(va, i)
+                        check_a = arrayimpl.scalar(i)
+                        self.assertEqual(va, check_a)
                     for vb in b:
-                        self.assertEqual(vb, -1)
+                        check_b = arrayimpl.scalar(-1)
+                        self.assertEqual(vb, check_b)
 
     def testAlltoallw2(self):
         size = self.COMM.Get_size()
         rank = self.COMM.Get_rank()
-        for array, typecode in arrayimpl.subTest(self):
+        for array, typecode in arrayimpl.loop():
             for count in range(size):
                 rbuf = array(-1, typecode, size*size)
                 for i in range(size):
@@ -365,9 +420,11 @@ class BaseTestCCOVecInplace(object):
                     row = rbuf[i*size:(i+1)*size]
                     a, b = row[:count], row[count:]
                     for va in a:
-                        self.assertEqual(va, i)
+                        check_a = arrayimpl.scalar(i)
+                        self.assertEqual(va, check_a)
                     for vb in b:
-                        self.assertEqual(vb, -1)
+                        check_b = arrayimpl.scalar(-1)
+                        self.assertEqual(vb, check_b)
 
 
 class TestCCOVecSelf(BaseTestCCOVec, unittest.TestCase):
