@@ -251,6 +251,8 @@ class BaseTestComm:
         self.assertEqual(comm, MPI.COMM_NULL)
         info.Set("mpi_hw_resource_type", "@dont-thread-on-me@")
         comm = self.COMM.Split_type(split_type, info=info)
+        if unittest.is_mpi('impi(==2021.14.0)'): comm.free()
+        if unittest.is_mpi('impi(==2021.14.1)'): comm.free()
         self.assertEqual(comm, MPI.COMM_NULL)
         info.Free()
         #
@@ -302,6 +304,8 @@ class BaseTestComm:
         self.assertEqual(comm, MPI.COMM_NULL)
         info.Set("mpi_hw_resource_type", "@dont-thread-on-me@")
         comm = self.COMM.Split_type(split_type, info=info)
+        if unittest.is_mpi('impi(==2021.14.0)'): comm.free()
+        if unittest.is_mpi('impi(==2021.14.1)'): comm.free()
         self.assertEqual(comm, MPI.COMM_NULL)
         info.Free()
         #
@@ -420,7 +424,7 @@ class TestCommSelf(BaseTestComm, unittest.TestCase):
         rank = self.COMM.Get_rank()
         self.assertEqual(rank, 0)
 
-    @unittest.skipMPI('openmpi(<5.2.0)')  # open-mpi/ompi#10895
+    @unittest.skipMPI('openmpi(<5.0.6)')
     def testCreateFromGroup(self):
         super().testCreateFromGroup()
 
