@@ -2,27 +2,25 @@ from mpi4py import MPI
 import mpiunittest as unittest
 import sys, os, mpi4py
 
-MPI4PYPATH = os.path.abspath(os.path.dirname(mpi4py.__path__[0]))
+MPI4PYPATH = os.path.abspath(
+    os.path.dirname(mpi4py.__path__[0])
+)
 
 CHILDSCRIPT = os.path.abspath(
     os.path.join(os.path.dirname(__file__), 'spawn_child.py')
-    )
-
-HAVE_MPE = 'MPE_LOGFILE_PREFIX' in os.environ
-HAVE_VT  = 'VT_FILE_PREFIX' in os.environ
+)
 
 def childscript():
     from tempfile import mkstemp
     from textwrap import dedent
     fd, script = mkstemp(suffix='.py', prefix="mpi4py-")
     os.close(fd)
+    python = sys.executable
+    pypath = MPI4PYPATH
     with open(script, "w") as f:
-        f.write(dedent("""\
-        #!%(python)s
-        import sys; sys.path.insert(0, "%(path)s")
-        import mpi4py
-        if %(mpe)s: mpi4py.profile('mpe', logfile="%(logfile)s")
-        if %(vt)s:  mpi4py.profile('vt',  logfile="%(logfile)s")
+        f.write(dedent(f"""\
+        #!{python}
+        import sys; sys.path.insert(0, "{pypath}")
         from mpi4py import MPI
         parent = MPI.Comm.Get_parent()
         parent.Barrier()
@@ -30,9 +28,7 @@ def childscript():
         assert parent == MPI.COMM_NULL
         parent = MPI.Comm.Get_parent()
         assert parent == MPI.COMM_NULL
-        """ % dict(python=sys.executable, path=MPI4PYPATH,
-                   mpe=HAVE_MPE, vt=HAVE_VT,
-                   logfile="runtests-mpi4py-child")))
+        """))
     os.chmod(script, int("770", 8))
     return script
 
@@ -61,41 +57,69 @@ def using_GPU():
     # Note: This seems to be a partial fix. Even if we are running cpu-only
     # tests, if MPI is built with CUDA support we can still fail. Unfortunately
     # there is no runtime check for us to detect if it's the case...
-    disabled_cupy = (sys.modules.get('cupy', -1) is None)
-    disabled_numba = (sys.modules.get('numba', -1) is None)
-    return False if (disabled_cupy and disabled_numba) else True
+    using_cupy = (sys.modules.get('cupy') is not None)
+    using_numba = (sys.modules.get('numba') is not None)
+    return using_cupy or using_numba
 
+def sequential():
+    return MPI.COMM_WORLD.Get_size() == 1
+
+def macos():
+    return sys.platform == 'darwin'
+
+def windows():
+    return sys.platform == 'win32'
+
+def github():
+    return os.environ.get('GITHUB_ACTIONS') == 'true'
+
+def azure():
+    return os.environ.get('TF_BUILD') == 'True'
+
+def skip_spawn():
+    return (
+        os.environ.get('MPI4PY_TEST_SPAWN')
+        in (None, '0', 'no', 'off', 'false')
+    )
 
 @unittest.skipMPI('MPI(<2.0)')
 @unittest.skipMPI('openmpi(<3.0.0)')
 @unittest.skipMPI('openmpi(==4.0.0)')
-@unittest.skipMPI('openmpi(==4.0.1)', sys.platform=='darwin')
-@unittest.skipMPI('openmpi(==4.0.2)', sys.platform=='darwin')
+@unittest.skipMPI('openmpi(==4.0.1)', macos())
+@unittest.skipMPI('openmpi(==4.0.2)', macos())
+@unittest.skipMPI('openmpi(>=4.1.0,<4.2.0)', azure())
+@unittest.skipMPI('openmpi(>=4.1.0,<4.2.0)', github())
+@unittest.skipMPI('openmpi(>=5.0.0,<5.0.7)', skip_spawn())
 @unittest.skipMPI('mpich(<4.1.0)', appnum() is None)
-@unittest.skipMPI('mpich(==4.1.2)', sys.platform=='darwin')
-@unittest.skipMPI('mpich', badport())
+@unittest.skipMPI('mpich(<4.3.0)', badport())
 @unittest.skipMPI('msmpi(<8.1.0)')
+@unittest.skipMPI('msmpi', skip_spawn())
 @unittest.skipMPI('msmpi', appnum() is None)
-@unittest.skipMPI('MVAPICH2')
+@unittest.skipMPI('msmpi', os.environ.get("PMI_APPNUM") is None)
+@unittest.skipMPI('mvapich', appnum() is None)
+@unittest.skipMPI('mvapich', badport())
+@unittest.skipMPI('mvapich(<3.0.0)')
 @unittest.skipMPI('MPICH2')
 @unittest.skipMPI('MPICH1')
-@unittest.skipMPI('PlatformMPI')
 @unittest.skipIf(using_GPU(), 'using CUDA')
-class BaseTestSpawn(object):
+class BaseTestSpawn:
 
     COMM = MPI.COMM_NULL
     COMMAND = sys.executable
-    ARGS = [CHILDSCRIPT, MPI4PYPATH,
-            "mpe" if HAVE_MPE else
-            "vt"  if HAVE_VT  else ""]
+    ARGS = [CHILDSCRIPT, MPI4PYPATH]
     MAXPROCS = 1
     INFO = MPI.INFO_NULL
     ROOT = 0
 
+
+class BaseTestSpawnSingle(BaseTestSpawn):
+
     def testCommSpawn(self):
         self.COMM.Barrier()
-        child = self.COMM.Spawn(self.COMMAND, self.ARGS, self.MAXPROCS,
-                                info=self.INFO, root=self.ROOT)
+        child = self.COMM.Spawn(
+            self.COMMAND, self.ARGS, self.MAXPROCS,
+            info=self.INFO, root=self.ROOT,
+        )
         local_size = child.Get_size()
         remote_size = child.Get_remote_size()
         child.Barrier()
@@ -108,9 +132,11 @@ class BaseTestSpawn(object):
     def testErrcodes(self):
         self.COMM.Barrier()
         errcodes = []
-        child = self.COMM.Spawn(self.COMMAND, self.ARGS, self.MAXPROCS,
-                                info=self.INFO, root=self.ROOT,
-                                errcodes=errcodes)
+        child = self.COMM.Spawn(
+            self.COMMAND, self.ARGS, self.MAXPROCS,
+            info=self.INFO, root=self.ROOT,
+            errcodes=errcodes,
+        )
         child.Barrier()
         child.Disconnect()
         self.COMM.Barrier()
@@ -123,11 +149,15 @@ class BaseTestSpawn(object):
     def testArgsOnlyAtRoot(self):
         self.COMM.Barrier()
         if self.COMM.Get_rank() == self.ROOT:
-            child = self.COMM.Spawn(self.COMMAND, self.ARGS, self.MAXPROCS,
-                                    info=self.INFO, root=self.ROOT)
+            child = self.COMM.Spawn(
+                self.COMMAND, self.ARGS, self.MAXPROCS,
+                info=self.INFO, root=self.ROOT,
+            )
         else:
-            child = self.COMM.Spawn(None, None, -1,
-                                    info=MPI.INFO_NULL, root=self.ROOT)
+            child = self.COMM.Spawn(
+                None, None, -1,
+                info=MPI.INFO_NULL, root=self.ROOT,
+            )
         child.Barrier()
         child.Disconnect()
         self.COMM.Barrier()
@@ -140,8 +170,10 @@ class BaseTestSpawn(object):
             script = childscript()
         self.COMM.Barrier()
         script = self.COMM.bcast(script, root=self.ROOT)
-        child = self.COMM.Spawn(script, None, self.MAXPROCS,
-                                info=self.INFO, root=self.ROOT)
+        child = self.COMM.Spawn(
+            script, None, self.MAXPROCS,
+            info=self.INFO, root=self.ROOT,
+        )
         child.Barrier()
         child.Disconnect()
         self.COMM.Barrier()
@@ -149,7 +181,10 @@ class BaseTestSpawn(object):
             os.remove(script)
         self.COMM.Barrier()
 
-    def testCommSpawnMultiple(self):
+
+class BaseTestSpawnMultiple(BaseTestSpawn):
+
+    def testCommSpawn(self):
         self.COMM.Barrier()
         count = 2 + (self.COMM.Get_size() == 0)
         COMMAND = [self.COMMAND] * count
@@ -158,7 +193,8 @@ class BaseTestSpawn(object):
         INFO = [self.INFO] * len(COMMAND)
         child = self.COMM.Spawn_multiple(
             COMMAND, ARGS, MAXPROCS,
-            info=INFO, root=self.ROOT)
+            info=INFO, root=self.ROOT,
+        )
         local_size = child.Get_size()
         remote_size = child.Get_remote_size()
         child.Barrier()
@@ -167,7 +203,7 @@ class BaseTestSpawn(object):
         self.assertEqual(local_size, self.COMM.Get_size())
         self.assertEqual(remote_size, sum(MAXPROCS))
 
-    def testCommSpawnMultipleDefaults1(self):
+    def testCommSpawnDefaults1(self):
         self.COMM.Barrier()
         count = 2 + (self.COMM.Get_size() == 0)
         COMMAND = [self.COMMAND] * count
@@ -181,7 +217,7 @@ class BaseTestSpawn(object):
         self.assertEqual(local_size, self.COMM.Get_size())
         self.assertEqual(remote_size, len(COMMAND))
 
-    def testCommSpawnMultipleDefaults2(self):
+    def testCommSpawnDefaults2(self):
         self.COMM.Barrier()
         count = 2 + (self.COMM.Get_size() == 0)
         COMMAND = [self.COMMAND] * count
@@ -196,7 +232,7 @@ class BaseTestSpawn(object):
         self.assertEqual(remote_size, len(COMMAND))
 
     @unittest.skipMPI('msmpi')
-    def testErrcodesMultiple(self):
+    def testErrcodes(self):
         self.COMM.Barrier()
         count = 2 + (self.COMM.Get_size() == 0)
         COMMAND = [self.COMMAND] * count
@@ -207,7 +243,8 @@ class BaseTestSpawn(object):
         child = self.COMM.Spawn_multiple(
             COMMAND, ARGS, MAXPROCS,
             info=INFO, root=self.ROOT,
-            errcodes=errcodelist)
+            errcodes=errcodelist,
+        )
         child.Barrier()
         child.Disconnect()
         self.COMM.Barrier()
@@ -218,7 +255,7 @@ class BaseTestSpawn(object):
                 self.assertEqual(errcode, MPI.SUCCESS)
 
     @unittest.skipMPI('msmpi')
-    def testArgsOnlyAtRootMultiple(self):
+    def testArgsOnlyAtRoot(self):
         self.COMM.Barrier()
         if self.COMM.Get_rank() == self.ROOT:
             count = 2 + (self.COMM.Get_size() == 0)
@@ -228,17 +265,19 @@ class BaseTestSpawn(object):
             INFO = [MPI.INFO_NULL] * len(COMMAND)
             child = self.COMM.Spawn_multiple(
                 COMMAND, ARGS, MAXPROCS,
-                info=INFO, root=self.ROOT)
+                info=INFO, root=self.ROOT,
+            )
         else:
             child = self.COMM.Spawn_multiple(
                 None, None, -1,
-                info=MPI.INFO_NULL, root=self.ROOT)
+                info=MPI.INFO_NULL, root=self.ROOT,
+            )
         child.Barrier()
         child.Disconnect()
         self.COMM.Barrier()
 
     @unittest.skipIf(os.name != 'posix', 'posix')
-    def testNoArgsMultiple(self):
+    def testNoArgs(self):
         self.COMM.Barrier()
         script = None
         if self.COMM.Get_rank() == self.ROOT:
@@ -249,8 +288,10 @@ class BaseTestSpawn(object):
         COMMAND = [script] * count
         MAXPROCS = list(range(1, len(COMMAND)+1))
         INFO = [self.INFO] * len(COMMAND)
-        child = self.COMM.Spawn_multiple(COMMAND, None, MAXPROCS,
-                                         info=INFO, root=self.ROOT)
+        child = self.COMM.Spawn_multiple(
+            COMMAND, None, MAXPROCS,
+            info=INFO, root=self.ROOT,
+        )
         child.Barrier()
         child.Disconnect()
         self.COMM.Barrier()
@@ -258,18 +299,45 @@ class BaseTestSpawn(object):
             os.remove(script)
         self.COMM.Barrier()
 
-class TestSpawnSelf(BaseTestSpawn, unittest.TestCase):
+    def testArgsBad(self):
+        if self.COMM.Get_size() > 1: return
+        CMDS = [self.COMMAND]
+        ARGS = [self.ARGS]
+        MAXP = [self.MAXPROCS]
+        INFO = [self.INFO]
+        with self.assertRaises(ValueError):
+            self.COMM.Spawn_multiple(CMDS[0], ARGS, MAXP, INFO, root=0)
+        with self.assertRaises(ValueError):
+            self.COMM.Spawn_multiple(CMDS, ARGS*2, MAXP, INFO, root=0)
+        with self.assertRaises(ValueError):
+            self.COMM.Spawn_multiple(CMDS, ARGS[0][0], MAXP*2, INFO, root=0)
+        with self.assertRaises(ValueError):
+            self.COMM.Spawn_multiple(CMDS, ARGS, MAXP[0], INFO*2, root=0)
+
+
+class TestSpawnSingleSelf(BaseTestSpawnSingle, unittest.TestCase):
     COMM = MPI.COMM_SELF
 
-class TestSpawnWorld(BaseTestSpawn, unittest.TestCase):
+class TestSpawnSingleWorld(BaseTestSpawnSingle, unittest.TestCase):
     COMM = MPI.COMM_WORLD
 
-class TestSpawnSelfMany(BaseTestSpawn, unittest.TestCase):
-    COMM = MPI.COMM_SELF
+class TestSpawnSingleSelfMany(TestSpawnSingleSelf):
     MAXPROCS = MPI.COMM_WORLD.Get_size()
 
-class TestSpawnWorldMany(BaseTestSpawn, unittest.TestCase):
+class TestSpawnSingleWorldMany(TestSpawnSingleWorld):
+    MAXPROCS = MPI.COMM_WORLD.Get_size()
+
+
+class TestSpawnMultipleSelf(BaseTestSpawnMultiple, unittest.TestCase):
+    COMM = MPI.COMM_SELF
+
+class TestSpawnMultipleWorld(BaseTestSpawnMultiple, unittest.TestCase):
     COMM = MPI.COMM_WORLD
+
+class TestSpawnMultipleSelfMany(TestSpawnMultipleSelf):
+    MAXPROCS = MPI.COMM_WORLD.Get_size()
+
+class TestSpawnMultipleWorldMany(TestSpawnMultipleWorld):
     MAXPROCS = MPI.COMM_WORLD.Get_size()
 
 
